/************************************************/

/*	BE Modelisation Geometrique	        */
/*		Loic Barthe			*/
/*	      IRIT - UPS - CNRS			*/

/************************************************/
/************************************************/
/************************************************/

/*	BE illustration des Blobs	        */

/************************************************/
/************************************************/


#ifndef __LISTE_POINTS__
#define __LISTE_POINTS__

#include "sumblob.h"
#include <GL/gl.h>


/**********************************************************************
    Les etudiants doivent coder la fonction recursive appelée dans :

                     creeListeTabPointsSumBlob

                  voir le fichier liste_points.c

**********************************************************************/


/***********************************************************/
/*	Liste simplement chainee de tableaux de points	   */
/* destinee au stockage des points, normales et courleurs  */
/*                  et a leur affichage                    */
/***********************************************************/


/* structure pour stocker un point avec sa normale et sa couleur */
typedef struct sPNC {
	float point[3];
	float normale[3];
	float couleur[3];
	} tPNC, *pPNC;
	

/* cellule stockant : 
	- un tableau de points 
	- les normales de ces points
	- leurs couleurs
	- le nombre de points
	- un tableau d'index destine a l'affichage OpenGL = {0,1,2,3,...,nbPoints-1} 
	- un pointeur vers la cellule suivante ( = NULL si il n'y en a pas) */
typedef struct sTabPoints tTabPoints,*pTabPoints;

struct sTabPoints {

	int nbPoints;
	float points [4096*3];
	float normales [4096*3];
	float couleurs [4096*3];
	GLuint index [4096];
	
	pTabPoints suivant;
	};
	
/* structure stockant :
	- le premier element de la liste 
	- l'element courant 
	- le nombre de points deja stockes */
typedef struct sListeTabPoints tListeTabPoints, *pListeTabPoints;

struct sListeTabPoints {
	
	pTabPoints premier;
	pTabPoints courant;
	
	int nbPoints;
};

/*******************************************/
/*	Methodes sur une cellule	   */
/*******************************************/

/* creation d'une cellule :
	- Allocation memoire du pointeur a retourner
	- initialisation du nombre de points
	- creation du tableau d'index destine a l'affichage 
	- cellule suivante = NULL */
pTabPoints creeTabPoints ();

/* ajoute un point, sa normale et sa couleur : le debordement du tableau 
sera gere au niveau de la liste et non au niveau de la cellule */
void ajoutePointTab (pTabPoints t, tPNC p);

/* affichage OpenGL d'un tableau de points avec un tableau de normales et 
un tableau de couleurs en utilisant les VERTEX_ARRAY */
void afficheTabPoints (pTabPoints t, float taillePoint);



/*******************************************/
/*	Methodes sur la liste    	   */
/*******************************************/

/* creation d'une liste : 
	- Allocation memoire du pointeur a retourner 
	- initialisation des differents champs
	- on considere que la liste contient une cellule non NULL a sa creation */
pListeTabPoints creeListeTabPoints ();

/* creation d'une liste a partir de la fonction potentiel d'une somme de blobs 
	- on utilise un decoupage recursif de l'espace en utilisabt l'arithmetique d'intervalle 
	- on decoupe l'espace "profondeur" fois et on stocke les centres des voxels secants a la 
	  surface. On calcule leur normale avec la fonction potentiel et eventuellement leur
	  couleur */ 
pListeTabPoints creeListeTabPointsSumBlob (pSum s, int profondeur, float iso, tIntervalle X, tIntervalle Y, tIntervalle Z);

/* Ajoute un point a une liste */
void ajoutePointListe (pListeTabPoints l, tPNC p);

/* lideration de la memoire utilisee par une liste */
void detruireListeTabPoints (pListeTabPoints l);

/* affichage d'une liste : appel de l'affichage OpenGL sur chaque cellule */
void afficheListeTabPoints (pListeTabPoints l, float taillePoint);

#endif
